##################################################################
##################################################################
# Gov 2002/E-2001 Introductory R Section
# 1-29-14
# Stephen Pettigrew
##################################################################
##################################################################

## This file walks you through the most basic R stuff
## Thanks to Sole Prillaman for letting me pull from their materials

## This code and data is available at: http://bit.ly/1fcCXaW

########################################################
## PREAMBLE: Script and Console
########################################################

## R works by 
## 1. typing code in the console 
## 2. OR by typing on a script (like this one) and sending the lines of
## code to the console.

## Unless you are working on very simple calculations,
## you should always save your code as a script using a ".R" file extension.

# Let's set up a script
# Open a new file in R editor, save it as "section1.R"
# Put commands in the script and execute them one by one

## Every line that begins with '#' is ignored by the console.
## So you can add comments to your code without generating syntax errors.
## It's good practice to comment everything in your code.
## Somebody who knows R should be able to take your code and understand
## exactly what it's doing, without you there to explain to them.

## Note: to send a line of code to your console, use either 
## (PC users:) "control + enter"
## (Mac users:) "command + return"


########################################################
## Introduction to R
########################################################

## R can function as a calculator

2+2

6*12

2^8 # ^ means to the power of

86/4

log(12) # the log function in R is based e, or the "natural log"

exp(2) ## equivalent to 2.71828^2 or e squared

2 * (6 + 2)^2 ##R preserves order of operations
2 * (8)^2  # equivalent to the expression above
2 * 64

2/9^2 ## another order of operations example
2/9^2 == 2/81 ## this statement is true
2/9^2 == (2/9) ^2 ## this one is false because you exponentiate before dividing


########################################################
## getting help in R
########################################################

## Let's say you want to get the mean of a vector...
## How could we do that?

## Let's say the vector is (86, 14, 9)

# we could calculate the mean by brute force
(86 + 14 + 9) / 3


## But what if the vector has 100 or 1,000,000 entries?
## Maybe there's a mean function in R
## Let's see...

## put two question marks in front of a word to look it up in the R help files
??mean

## if we already know the name of the function we want to learn about,
## we can look it up using a single question mark
?mean

## or, if you are using RStudio, there is a help tab in the bottom-right quadrant
## you can use the search bar to directly search the help files

## so now let's try to take the mean of our vector
mean(86, 14, 9) ## that gives us the wrong answer, so the code is wrong!

## We must have used the wrong syntax.
## The "Arguments" section of the mean help file tells us that
## x has to be an R object that's a "numeric/logical vector."
## But what's that actually mean?


########################################################
## objects and vectors
########################################################

## R is an object-oriented programming language.
## This means that you will store information (vectors, functions, data, etc) as objects
## to do so, use " <- " as the assignment operator.

## First, let's create our 86, 14, 9 vector

## we use the "combine" or "concatenate" function to put elements into a vector.
## you can name your vector whatever you want, so long as the name does not begin with a number
example.vector <- c(86, 14, 9)
example.vector

## Now let's try the mean function again.
mean(example.vector)
(86 + 14 + 9) / 3 ## they give us the same answer! 

## we can even store that mean as its own object (a constant)
vec.mean <- mean(example.vector)
vec.mean

## Now let's store some other objects to our workspace
minutes <- 60 * 24 * 365 # numeric object that's the number of minutes in a year
minutes

## Objects don't have to be numeric, they can be character strings (letters/words)
name <- "Sole" # use "" to tell R that the object is a character rather than a numeric
name

mean(name) ## this won't work, since the "name" object isn't numeric


## We can reassign what's in an object, but it overwrites the original object
name
name <- c("Stephen", "Pettigrew")
name ## this object is called a "character vector,"
## since it's literally a vector (of length 2) of character elements


## we saw that you can combine elements in a vector by concatenating them
## you can also create a sequence vector using the seq function
two.to.ten <- seq(from = 2, to = 10, by = 1)
	# need to fill in three arguments for seq()
  # the help file will show us what these are
?seq # the start value for the sequence, the end value, and the increment
two.to.ten

## you can also create a vector with the same item repeated using the rep function
five.fours <- rep(4, times=5)
five.fours

## you can even combine the above methods
combine.vec <- c(86, 14, 9, seq(2,10,1), rep(4,5))
combine.vec


########################################################
########################################################
### Work Break
########################################################
########################################################


# Create a vector containing all of the integers from 100 to 200 and store it
# To an object named "seq12"

# Then store the mean of that vector to an object named mean12

















seq12 <- seq(100,200,1)
seq12
mean12 <- mean(seq12)
mean12


#############################################
## matrices: two dimensional (row x column) object
#############################################

## We have already learned how to deal with vectors
## Another type of object that we use a lot are matrices (two-dimensional)
## Hint: It's row by column (not column by row)
## because it's remote controlled (RC) cars, RC Cola,
## roller coaster, Roman Catholic, and Ray Charles

## Because if you mix things up and you make your matrix with
## dimension Charles Ray, you'll be doing statistics blindly


## let's put a vector into a 2x3 matrix
## first create the vector
vector1 <- seq(1,6,1)

## now let's make a matrix
## the function we want to use is matrix()
matrix1 <- matrix(vector1, nrow = 2, ncol = 3)
matrix1

## One thing to remember about matrices in R is that the data
## you put in them fills the matrix column by column, top to bottom
## What if we want to fill the matrix to look like this:
## 1 2 3
## 4 5 6
## 7 8 9
matrix(1:9, nrow = 3, ncol = 3, byrow=T)


########################################################
## subsetting objects
########################################################

## Putting brackets, [], at the end of an object can pull out specific elements
## of that object (subset it).
## Let's say we want to get the third element of a particular vector:

seq12[3]

## Or we can pull out multiple elements, such as the 3rd and 7th
seq12[c(3,7)]

## We can even use brackets to reassign values within an object, without
## having to make a totally new object:
## We see that the third element of vector1 equals 6. Let's change is to -13

seq12[3] <- -13
seq12



## To pull out elements from a matrix, you must specify the column and row number for the element
## Remember that vectors are one dimensional, while matrices are two dimensional.
## To extract an element from a matrix use the follow structure of your code:
## matrix[row, column], where the row number and column numbers are called
## indexes.

## So if we want to see the third row, fourth column of mult.matrix:
matrix1[2,3]

## If we leave the row number blank, but specify a column, R will give us
## a vector of all the values in that column
matrix1[,2]
matrix1[2,]

## What happens if we forget the comma?
## R outputs the 2nd element of the matrix.
## This is a really common mistake. Don't let it screw you up
matrix1[2]


########################################################
## workspace
########################################################

## Objects are stored in what we call the "workspace."
## If you're using RStudio, you can see what's in your workspace on the top right.
## If not, use the ls() function to get a list of everything in the workspace.

ls()


#######################################################
## Working Directories
#######################################################

## The most common thing we want to do in R is work with data of some sort.
## Usually these data are in some format like .csv (excel spreadsheets),
## .dta (Stata dataset), or .txt (text files).

## Reading these into R (i.e. adding them to your workspace) is very simple,
## but it first requires telling R where they are on your computer.

## We do this by setting our working directory, which is the folder in which
## the files are stored. If you try to load data into R which is not inside
## the folder which is specified in your working directory, you'll get an error.
## Also, anytime you export anything from R (new datasets, graphs, etc.) it will
## save them to your working directory

## To see your current working directory, use the getwd() function
getwd()

## I want to change the working directory to my "Section 1" folder, so
## I'll use the setwd() function

setwd("C:/Users/Stephen/Dropbox/Gov2001 2014/Spring2014/Week 1 - 2014-01-27/R section")

## note that I wrapped the directory path in quotation marks
## If you don't do that, R will assume you're referring to an object in your
## workspace in R, not a folder on your computer

########################################################
## working with data
########################################################

## Next we need to call our data into R
## There are three basic filetypes that you typically would want to
## read into R: .RData (which is data created by R),
## .csv or .txt (which is data stored in a table format, maybe created by Excel),
## and everything else (Stata, SAS, SPSS, shapefiles, etc.).

## Note that the datafile houses.txt should be saved in your working directory
## Let's open it
data <- read.table("houses.txt", head=T)
ls() # you can see that we have added data to our workspace

## When working with .txt data, use the command read.table
## when working with R data, use the command load()
## when working with stata data, use the command read.dta() in conjunction with the foreign package
## More on packages later

## What if we want to observed our data?
## We can look at the first six elements of our data
head(data)
## or the first 10
head(data, 10)

View(data)

## We can see the names of all of the variables in our dataset
names(data)
colnames(data)

## And we can verify that our object data is actually a dataframe
## The class function will tell you the type of object
class(data)


## Like when we were working with vectors, we can assign new objects from old objects
## by subsetting the data (same as with matrix)
price <- data[,1]
price

## We can also call the price variable out directly from the data
## $ specify variables within data frames
data$PRICE



########################################################
########################################################
### Work Break
########################################################
########################################################

# Take the mean of the features variable in the dataset (FEATS)



















feats <- data$FEATS
mean(feats)
## or in one line:
mean(data$FEATS)

mean(data[,4])
mean(data[,"FEATS"])



########################################################
### graphing data
########################################################

## Say we want to subset our dataset to include only the observations (houses)
## which are corner hourses
corner.houses <- data[data$COR == 1,]
corner.houses <- subset(data, data$COR == 1)

## Then we want to plot a histogram of this new data
hist(corner.houses$PRICE)

## To learn ways to make your histogram prettier, check the help file
?hist


## What if we instead want to see the correlation between squarefoot and price
## We can use a scatterplot to do this
plot(data$SQFT, data$PRICE)
## Notice that the second variable we list is placed on the y-axis
## and the first variable we list is placed on the x-axis
## We could also have made this more specific

pdf("graph1.pdf")
plot(x=data$SQFT, y=data$PRICE,
     xlab = "Square Feet",
     main = "title")
graphics.off()
#dev.off()


########################################################
########################################################
### Work Break
########################################################
########################################################

# Make a histogram of the square footage of houses.





















hist(data$SQFT, xlab="Square Footage", main = "Histogram of Housing Square Footage", col="cornflowerblue")


  




#########################################################################
#########################################################################
#########################################################################
#########################################################################
#########################################################################
#### Functions
#########################################################################
#########################################################################
#########################################################################
#########################################################################
#########################################################################

## One of the most powerful parts of R is how easy it is write your own
## functions.

## Let's imagine that there wasn't a canned way to take a mean of a vector of
## numbers. We could easily write a function that does this

our.list <- list(minutes, seq12, vec.mean, vector1)


our.mean.function <- function(x){
  
  ## This part gets the sum of all the numbers in the x vector
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
  
  results <- list(our.sum, our.mean)
  
  return(results) 
}


final <- our.mean.function(1:10)

## Notice that within the function we assigned objects to the
## names 'our.sum' and 'our.mean', but we can't see what those equal

our.sum ## not found
our.mean ## not found


## This is because the only thing that a user-defined function outputs
## is what is in the return() function at the end. 


## Alternatively, we don't necessarily need to return() function...

our.mean.function <- function(x){
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
  
  our.mean
}

our.mean.function <- function(x){
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
}

our.mean.function(1:10)

## Notice that this function doesn't return anything, but if we store it as
## an object it will work.

a <- our.mean.function(1:10)
a



## Another way we can return the function out put is by not storing anything
## in the last line of code of the function.

our.mean.function <- function(x){
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
  
  our.mean
}

our.mean.function(1:10)




## Write a function that takes a vector of numbers and returns the 
## cube root divided by the natural log of each number in the vector




















example <- function(x){
  x^(1/3) / log(x)
}

example(5:10)





########################################
## Another example

# Let's say you have a bag that's filled with 50 red and 50 blue
# balls. You randomly select 10 balls, and you want to know
# how many were red and how many were blue.

# Let's write a function to do that.

# First set up my bag:

my.bag <- c(rep("blue", 50), rep("red", 50))
my.bag

# Now write a function that draws 10 balls from the bag
# and outputs how many red and how many blue there were

colors <- function(bag){
  
  selection <- sample(bag, 10, replace = F)
  
  reds <- sum(selection == "red")
  blues <- sum(selection == "blue")

  results <- c(blues,reds)
  names(results) <- c("blues","reds")
  
  return(results)
}

colors(my.bag)


# Or we could be even more efficient with our code by
# using the table() function:

colors <- function(bag){
  
  selection <- sample(bag, 10, replace = F)
  
  table(selection)
  
}

colors(my.bag)

  
  
  

###########################################################
###########################################################
###########################################################
###########################################################
## For loops
###########################################################


## A "for-loop" is a way of repeating a process a vast 
## number of times.  For each iteration of the loop
## R keeps an index number stored which can be handy.


#######################################
## Indexing in loops
#######################################

## To understand loops in any programming language, you first
## need to understand indexing in a loop

## Take this very simple loop:
## In this case, the loop is indexed by 'i'.
## The loop starts with i = 1, goes through all the lines of the loop
## then returns to the top of the loop and sets i = 2. The loop knows to
## stop when it reaches i = 10

for(i in 1:10){
  
  print(i)
  
}

## That loop's indexing is identical to this one:

for(i in seq(1,10)){
  
  print(i)
  
}


## And this one:

index <- 1:10

for(i in index){
  
  print(i)
  
}





## Your indexes don't need to be sequential
# This one prints all even numbers between 2 and 50

for(i in seq(2,50,2)){
  
  print(i)
}


## Side note: "print()" statements are an incredibly useful
## thing to put into code. They'll help you debug your code by letting
## you know where your code hit snags. Or if you're looping over a huge number
## of things that take a really long time, they'll tell you where to restart
## your loop if you turn off your computer






## Your loop doesn't even need to be indexed by numbers:
index <- c("look", "how", "fancy", "we", "can",
           "be", "with", "loop", "indexes")

for(i in index){
  
  print(i)
  
}





## Another note about indexing and loops:
## If you put loops within loops, be careful that you never use the 
## the same letter to index two different loops.

## In other words, don't do this:

for(i in 1:10){
  
  for(i in 50:60){
    
    ## Some sort of code...
  }
  
}

## Don't this won't necessarily prevent your code from working,
## but it could cause you a headache that can be frustrating to 
## try to diagnose. 

## Instead your loops should look something like this:

for(i in 1:10){
  
  for(j in 50:60){
    
    ## some sort of code...
  }
  
}







## Remember our colors() function we wrote earlier?
## Let's say we want to take 1000 repeated samples from the
## bag of 50 red and 50 blue balls, and get a count each time
## of how many blue balls we drew

## How could we do this with a for loop?

## First, how we we pull out just the number of blues from
## the results of the function?

colors(my.bag) ## that has the blue and red counts
colors(my.bag)[1] ## this just has the first count, which is blue






## So let's see what happens if we want to run this code:

for(i in 1:1000){
  colors(my.bag)[1]
}







## We can force it to print the number of blues by using the
## print() function:

for(i in 1:1000){
  print(colors(my.bag)[1])
}



## But what if we want to do something with the results of our for loop,
## more that just have them be printed?

results <- c() # create an empty vector to store our results in

for(i in 1:1000){
  results[i] <- colors(my.bag)[1]
}

results
mean(results)
sd(results)